﻿using Bios.Communication.Packets.Outgoing.Catalog;
using Bios.Communication.Packets.Outgoing.Messenger;
using Bios.Database.Interfaces;
using Bios.HabboHotel.GameClients;

namespace Bios.HabboHotel.Rooms.Chat.Commands.User
{
    class GroupChatCommand : IChatCommand
    {
        public string PermissionRequired => "command_groupchat";
        public string Parameters => "<ON/OFF>";
        public string Description => "De-/aktiviere den Gruppenchat.";

        public void Execute(GameClient Session, Room Room, string[] Params)
        {
            if (Params.Length < 2)
            {
                Session.SendWhisper("Lutfen bundan birink ullan (:grupchat <ON/OFF>).");
                return;
            }

            if (!Room.CheckRights(Session, true))
            {
                Session.SendWhisper("Bu komudu kullanmak icin iznin yok.");
                return;
            }

            if (Room.Group.CreatorId != Session.GetHabbo().Id)
            {
                Session.SendWhisper("Sen grubun kurucusu degilsin.");
                return;
            }

            if (Room.Group == null)
            {
                Session.SendWhisper("Bu odada grup yok.");
                return;
            }

            if (Room.Group.Id != Session.GetHabbo().GetStats().FavouriteGroupId)
            {
                Session.SendWhisper("Grup sohbetini etkinleştirmek / devre dışı bırakmak için grubunuza yer işareti koyun.");
                return;
            }

            var mode = Params[1].ToLower();
            var group = Room.Group;

            if (mode == "on")
            {
                if (group.HasChat)
                {
                    Session.SendWhisper("Grup konusmasi artik atkif.");
                    return;
                }

                group.HasChat = true;

                using (IQueryAdapter dbClient = BiosEmuThiago.GetDatabaseManager().GetQueryReactor())
                {
                    dbClient.SetQuery("UPDATE groups SET has_chat = '1' WHERE id = @id");
                    dbClient.AddParameter("id", group.Id);
                    dbClient.RunQuery();
                }

                var Client = BiosEmuThiago.GetGame().GetClientManager().GetClientByUserID(Session.GetHabbo().Id);
                if (Client != null)
                {
                    Client.SendMessage(new FriendListUpdateComposer(group, 1));
                }

            }
            else if (mode == "off")
            {
                if (!group.HasChat)
                {
                    Session.SendWhisper("Grup konusmasi artik devredisi.");
                    return;
                }

                group.HasChat = false;

                using (var adap = BiosEmuThiago.GetDatabaseManager().GetQueryReactor())
                {
                    adap.SetQuery("UPDATE groups SET has_chat = '0' WHERE id = @id");
                    adap.AddParameter("id", group.Id);
                    adap.RunQuery();
                }
                var Client = BiosEmuThiago.GetGame().GetClientManager().GetClientByUserID(Session.GetHabbo().Id);
                if (Client != null)
                {
                    Client.SendMessage(new FriendListUpdateComposer(group, -1));
                }
            }
            else
            {
                Session.SendNotification("Ein Fehler ist aufgetreten.");
            }
            

        }
    }
}
